Html Elements 1.9 – What's New?
===============================

While developing a new version of our framework we mainly worked on the drawbacks that were found by the members our 
community. We have fixed some architectural problems and a couple of bugs related to initialization of 
`List<Т extends HtmlElements>`.

And we've got a new contributors as well! Thanks a lot to [@lanwen](http://github.com/lanwen) for the great 
"WaitFor" matcher implementation. We'd also like to thank [@pazone](http://github.com/pazone) for the idea of the matcher 
which recursively checks that all inner elements of a block exsist on a page, though the pull-request is still unaccepted. 
We hope it will be included in the next release.

If you still haven't got your "thank you" from [Yandex QATools](https://github.com/yandex-qatools) team, feel free to send 
us your ideas and your code as well.

And now it's time for a more detailed review of the new features.

Default usage of AjaxElementLocator
-----------------------------------

[Issue #3](https://github.com/yandex-qatools/htmlelements/issues/3)

This is one of the main and most important features of this release. Now `AjaxElementLocator` will be used by default 
instead of `DefaultElementLocator` for elements initialization . It allows to search element not immediately after its 
method call but after a time-out has expired (5 seconds by default). The name `Ajax...` explains its destination completely.

Ability to use your own ElementLocator
--------------------------------------

[Issue #4](https://github.com/yandex-qatools/htmlelements/issues/4)

We've also added the ability to use your own implementation of `Element Locator` for elements and page objects 
initialization:

    HtmlElementLoader.populateHtmlElement(HtmlElement htmlElement, CustomElementLocatorFactory locatorFactory)
    HtmlElementLoader.populatePageObject(Object page, CustomElementLocatorFactory locatorFactory)

"WaitFor" Matcher Decorator
---------------------------

[Issue #10](https://github.com/yandex-qatools/htmlelements/pull/10)

When dealing with client-side JavaScript we need to use a lot of different time-outs in our tests.<br/>
Suppose you need to check suggest appearance on search request typing on the [Yandex homepage](http://www.yandex.com).
Problem is that suggest is loaded with the help of AJAX, so this check has to be made after a time-out.<br/>
We've implemented "WaitFor" matcher to simplify such verifications. You can decorate any other matcher with it
in order to perform a check after a time-out. Here is an example:

    MainPage mainPage = new MainPage(driver);
    mainPage.getSearchArrow().getRequestInput().sendKeys(REQUEST);
    assertThat(mainPage.getSuggest(), withWaitFor(exists()));

In this case the exists() matcher will being executed every 500 ms (it's a default interval) until it return true or
30000 ms time-out (it's a default time-out) expires. In case the check not passes you'll get the following message:

    Expected: While waiting [<30000L>] millis it should be: element existing on page
         but: element Suggest not existing on page
         
Of course, you can define your own time-out instead of default one:

    int timeoutInMilliseconds = 10000; //10 seconds

    MainPage mainPage = new MainPage(driver);
    mainPage.getSearchArrow().getRequestInput().sendKeys(REQUEST);
    assertThat(mainPage.getSuggest(), withWaitFor(exists(), timeoutInMilliseconds));

And even the interval of decorated matcher execution:

    int timeoutInMilliseconds = 10000; //10 seconds
    int intervalInMilliseconds = 50; //50 milliseconds

    MainPage mainPage = new MainPage(driver);
    mainPage.getSearchArrow().getRequestInput().sendKeys(REQUEST);
    assertThat(mainPage.getSuggest(), withWaitFor(exists(), timeoutInMilliseconds, intervalInMilliseconds));

A really great matcher, isn't it?


"Refresh" Matcher Decorator
---------------------------

[Issue #19](https://github.com/yandex-qatools/htmlelements/pull/19)

Sometimes it is needed to perform some check after a page refresh. For example, it is necessary in case a request is 
sent to a server asynchronously, but the result of its execution comes into effect only after a page reload. Or you may 
also need to check that some page element changes after every page refresh (e.g. an advertising banner, captcha, etc.).<br/>
You can easy perform such checks with the help of "Refresh" matcher. It decorates any other matcher and refreshes current 
page before a check execution.<br/>
For example, let's verify, that search request example on [Yandex home page](http://www.yandex.ru) changes on page reload:

    MainPage mainPage = new MainPage(driver);
    String currentText = mainPage.getSearchSample().getText();
    assertThat(mainPage.getSearchSample(), withPrerefresh(not(hasText(currentText)), driver));

In this case `not(hasText(...))` check will be executed only after a page refresh. In case the check not passes
you'll get a comprehensive message:

    Expected: (after refresh) not element with text: who is Platon
         but: was <element with text: who is Platon>

Also this matcher is very useful in combination with "WaitFor" matcher. Suppose you need to check that a letter 
is delivered during the given time period after it was sent. It can be done very simply:

    assertThat(mailPage.getMailsList(), withWaitFor(withPrerefresh(hasMailWithSubject(SUBJECT), driver)));

Attempts to find a letter will being made until it is received or a timeout expires. 
And every attempt will be started with page refresh.

List&lt;T extends HtmlElement&gt; initialization bugfix
-------------------------------------------------------

[Issue #17](https://github.com/yandex-qatools/htmlelements/issues/17)

In the previous version there was a bug in a list of HtmlElements (or list of TypifiedElements) initialization. 
All list items had been compared with each other due to a bad items naming algorithm. As a result, HtmlElement list
initialization was quite slowly.<br/>
We've fixed the list items naming algorithm and now it works fine.

TypifiedElements useful methods
-------------------------------

[Issue #15](https://github.com/yandex-qatools/htmlelements/issues/15)

Turned out that `isDisplayed()`, `isEnabled()`, `isSelected()` methods are pretty useful for all types of elements.
So we've added them to the base TypifiedElements class.

Checkbox implementation improvements
------------------------------------

[Issue #14](https://github.com/yandex-qatools/htmlelements/issues/14)

The typified elements concept is quite immature yet, but we'll work on it in the next release.<br/>
In previous version there was an architectural drawback in Checkbox implementation:

    public void select() {
        if (!getWrappedElement().isSelected()) {
            getWrappedElement().click();
        }
    }

    public void deselect() {
        if (getWrappedElement().isSelected()) {
            getWrappedElement().click();
        }
    }

    public boolean isSelected() {
        return getWrappedElement().isSelected();
    }

So if you wanted to override `isSelected()` method, you had had to override the rest methods as well. It was inconvenient, 
so we've improved the `select()` and  `deselect()` methods to use isSelected() method:

    public void select() {
        if (!isSelected()) {
            getWrappedElement().click();
        }
    }


HtmlElements All
----------------

[Issue #18](https://github.com/yandex-qatools/htmlelements/issues/18)

We've made a separate module which contains the latest versions of htmlelements-java and htmlelements-matchers modules.
So you can use only one dependency instead of two:

    <dependency>
        <groupId>ru.yandex.qatools.htmlelements</groupId>
        <artifactId>htmlelements-all</artifactId>
        <version>1.9</version>
    <dependency>

As before, you can use htmlelements-java or htmlelements-matchers module separately.

Selenium Version update
-----------------------

[Issue #9](https://github.com/yandex-qatools/htmlelements/issues/9)

Selenium version was upgraded to 2.29.0

Java version downgrade 
----------------------

[Issue #7](https://github.com/yandex-qatools/htmlelements/issues/7)

The previous version of our framework was built using Java 1.7. 
Considering the fact that Java 1.6 is still widespread, we've downgraded Java version to 1.6.

___

That is all for the 1.9 version. But new features will be developed soon! And now it's time to go :)
